/*
 * DisplayImage.cpp
 *
 *  Created on: Apr 1, 2014
 *      Author: mseyed
 */

#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/ml/ml.hpp"
#include <dirent.h>
#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <iostream>
#include <algorithm>
#include <cmath>
#include "boost/filesystem.hpp"
#include "seriesClassifier.h"
#include "seriesClassifierMS.h"

using namespace std;
using namespace cv;

vector<string> listOfImages(const char* folder){
	// it returns a vector of the file names in the folder.
	vector<string> listOfFiles;
	DIR *dir;
	string path = folder;
	struct dirent *ent;
	if ((dir = opendir(folder)) != NULL) {
		int k = 0;
		while ((ent = readdir (dir)) != NULL) {
			//printf ("%s\n", ent->d_name); // print all the files and directories within directory
			k++;
			if (k>2) { // to avoid "." and "..".
				listOfFiles.push_back(ent->d_name);
			}
		}
		closedir (dir);
	}
	else {
		// Could not open directory
		printf("Sorry! could not open the directory!");
		perror("");
		//return EXIT_FAILURE;
	}
	return listOfFiles;
}


int main( int argc, char** argv )
{
  int flag = -1;
  char *Tpath, *Lpath, *Opath, *Tepath;
  string line, name;
  double eps;
  ifstream paramFile;
  int maxiter, nst, nhid, rad, multiscale, radc, nsc;
  if ( argc == 5 ) {
	  Tpath = argv[1];
	  Lpath = argv[2];
	  Opath = argv[3];
	  Tepath = argv[4];
	  flag = 1;
  }
  else if ( argc == 4 ) {
	  Tpath = argv[1];
	  Lpath = argv[2];
	  Opath = argv[3];
	  flag = 2;
  }
  else if ( argc == 3 ) {
	  Opath = argv[2];
	  Tepath = argv[1];
	  flag = 3;
  }
  else {
	  printf("Wrong number of input arguments");
	  return -1;
  }


  // Read the parameters from the "params.txt" file.
  paramFile.open("params.txt");

  while(getline(paramFile, line)) {
	stringstream parse(line);
	getline(parse, name, ':');
	if (name=="epsilon") {
		getline(parse, name, '\n');
		eps = atof(name.c_str());
	}
	else if (name=="maxiter") {
		getline(parse,name, '\n');
		maxiter = atof(name.c_str());
	}
	else if (name=="nstage") {
		getline(parse,name, '\n');
		nst = atof(name.c_str());
	}
	else if (name=="nhid") {
		getline(parse,name, '\n');
		nhid = atof(name.c_str());
	}
	else if (name=="rad") {
		getline(parse,name, '\n');
		rad = atof(name.c_str());
	}
	else if (name=="multiscale") {
		getline(parse,name, '\n');
		multiscale = atof(name.c_str());
	}
	else if (name=="radc") {
		getline(parse,name, '\n');
		radc = atof(name.c_str());
	}
	else if (name=="nsc") {
		getline(parse,name, '\n');
		nsc = atof(name.c_str());
	}
  }
	cout << "epsilon is: " << eps << endl;
	cout << "maximum iteration is: " << maxiter << endl;
	cout << "No. of stages is: " << nst << endl;
	cout << "No. of hidden nodes is: " << nhid << endl;
	cout << "radius of input patch is: " << rad << endl;
	cout << "multiscale mode is: " << multiscale << endl;
	cout << "radius of context for multiscale is: " << radc << endl;
	cout << "No. of scales in multiscale mode is: " << nsc << endl;



  //---------------------------------------- Run series single scale

  if (!multiscale) { // Single scale -> worse results
    seriesClassifier S;
    boost::filesystem::create_directories(Opath);
    S.setParameters(Tpath, Lpath, Opath, rad, nst, maxiter, nhid, eps);
    if ( flag == 1 || flag == 2 ) {
	    S.trainClassifierSeries();
    }
    if ( flag == 1 || flag == 3 ) {
	    S.generateResultsTest(Tepath);
    }
  }
  else { // Multi-scale -> faster and better results

    seriesClassifierMS S;
    boost::filesystem::create_directories(Opath);
    S.setParameters(Tpath, Lpath, Opath, rad, nst, maxiter, nhid, eps, nsc, radc);
    if ( flag == 1 || flag == 2 ) {
	    S.trainClassifierSeries();
    }
    if ( flag == 1 || flag == 3 ) {
	    S.generateResultsTest(Tepath);
    }
  }
  //-----------------------------------------

  cout << "****** DONE ******" << endl;
  cout << "CHECK " << Opath << " FOR RESULTS" << endl;


  return 0;
}
