/*
 * seriesClassifier.cpp
 *
 *  Created on: Apr 1, 2014
 *      Author: mseyed
 */
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/ml/ml.hpp"
#include <dirent.h>
#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include<boost/filesystem.hpp>
using namespace std;
using namespace cv;
#include "seriesClassifier.h"
vector<string> listOfImages(const char* folder);

seriesClassifier::seriesClassifier() {
	stage = 0;
	epsilon = 0.0001;

}

seriesClassifier::~seriesClassifier() {

}

// It extract patches from the context image.
void seriesClassifier::extractPatchContext(const Mat& img, Mat& trainData) {
	int counter = 0;
	Point2f center;	
	Mat patch(2*radius+1,2*radius+1,CV_32FC1);
	Size_<int> patchSize(2*radius+1,2*radius+1);
	trainData.create(img.cols * img.rows,(2*radius+1)*(2*radius+1),CV_32FC1);
	for (int i = 0; i < img.rows; ++i){
		for (int j = 0; j < img.cols; ++j){
			center.x = j;
			center.y = i;
			getRectSubPix(img, patchSize, center, patch); // extract patch around center pixel
			patch = patch.reshape(0,1);
			patch.row(0).copyTo(trainData.row(counter++));
		}
	}
}

// It gets an image and extract a patch around each pixel. It put the extracted patches in the rows of trainData matrix
void seriesClassifier::extractPatch(const Mat& img, Mat& trainData) {
	int counter = 0;
	Point2f center;
	Mat floatImg;
	img.convertTo(floatImg,CV_32F); // convert from uchar to float
	floatImg = floatImg / 255.0; // scale the image to [0 1]
	Mat patch(2*radius+1,2*radius+1,CV_32FC1);
	Size_<int> patchSize(2*radius+1,2*radius+1);// The size of the patch is 2*radius +1
	trainData.create(img.cols * img.rows,(2*radius+1)*(2*radius+1),CV_32FC1);
	for (int i = 0; i < floatImg.rows; ++i){
		for (int j = 0; j < floatImg.cols; ++j){
			center.x = j;
			center.y = i;
			getRectSubPix(floatImg, patchSize, center, patch); // extract patch around center pixel
			patch = patch.reshape(0,1);
			patch.row(0).copyTo(trainData.row(counter++));
		}
	}

}

// It puts the label image in trainLabel matrix (simple reshape). It prepares the format acceptable by mlp.train
void seriesClassifier::extractLabels(const Mat& label, Mat& trainLabel){
	label.copyTo(trainLabel);
	trainLabel = trainLabel > 127; // make it binary
	trainLabel.convertTo(trainLabel,CV_32F);
	trainLabel = trainLabel / 255.0; // scale it
	trainLabel = trainLabel.reshape(0,label.rows*label.cols);
}

// It trains the ANN classifier for each stage
void seriesClassifier::trainClassifier() {
	Mat layer(3,1,CV_32SC1);
	Mat image, label, train, D, context, contextF;
	vector<int> r; // shuffle vector
	setANNParam();
	stringstream sname;
	sname << pathO << "trainingOutputsStage" << stage-1 <<"/";
	string name  = sname.str();
	string sub;

	cout << "start learning classifier stage " << stage << endl;
	vector<string> trainI = listOfImages(trainP); // get the list of training images
	vector<string> labelI = listOfImages(labelP); // get the list of testing images

	// number of hidden nodes in each layer
	if (stage > 0) {
		layer.row(0) = inputNC;
	}
	else {
		layer.row(0) = inputN;
	}

	layer.row(1) = hiddenN;
	layer.row(2) = 1;



	ann.create(layer, CvANN_MLP::SIGMOID_SYM, 1, 1);
	int flags = 0;

	for (int i=0; i<trainI.size(); ++i) r.push_back(i);// initialize shuffle vector



	for (int k = 0; k < maxIter; ++k ){
		random_shuffle ( r.begin(), r.end() ); // shuffle the order of images in the training set.
		for (int i = 0; i < trainI.size(); ++i){
			image = imread(trainP + trainI[r[i]], CV_LOAD_IMAGE_GRAYSCALE);
			label = imread(labelP + labelI[r[i]], CV_LOAD_IMAGE_GRAYSCALE);
			extractPatch(image, train);
			if (stage > 0) {
				sub = trainI[r[i]].substr(0,trainI[r[i]].length()-4);
				sub += ".yml";
				FileStorage file(name + sub, FileStorage::READ);
				file["result"] >> context;
				file.release();
				extractPatchContext(context, contextF);
				hconcat(train, contextF, train);
			}


			extractLabels(label, D);
			if (k != 0 || i != 0){
				flags = CvANN_MLP::UPDATE_WEIGHTS;
			}
			ann.train( train, D, Mat(), Mat(), params, flags );

		}
		cout << "iteration " << k << " stage " << stage <<" ...done" << endl;
	}
	cout << "training stage " << stage << " ...done" << endl;
	saveANN();
}

// It computes the labels for a given image at each stage. For the first stage where there is no context image the second argument should be empty.
Mat seriesClassifier::testClassifier(const Mat& image, const Mat& context) {
	Mat testIn;
	extractPatch(image, testIn);

	if (stage > 0) {
		Mat contextF;
		extractPatchContext(context, contextF);
		hconcat(testIn, contextF, testIn);
	}

	Mat output(testIn.rows,1,CV_32FC1);
	ann.predict(testIn, output);
	output = output.reshape(0,image.rows);
	return output;
}

// It runs the trained classifier on the training images at each stage. The output of this method is used by the next stage.
void seriesClassifier::generateResults() {
	// Where data shoul be written
	stringstream WriteName;
	WriteName << pathO << "trainingOutputsStage" << stage <<"/";
	string writeName  = WriteName.str();
	boost::filesystem::create_directories(writeName);

	// Where context images shoud be read from.
	stringstream ReadName;
	ReadName << pathO << "trainingOutputsStage" << stage-1 <<"/";
	string readName  = ReadName.str();


	char* pathI = trainP;
	cout << "generating training results stage " << stage << endl;
	vector<string> inp = listOfImages(pathI);
	Mat image, context;
	Mat output;
	string sub;
	for (int i = 0; i < inp.size(); ++i) {
		image = imread(pathI + inp[i], CV_LOAD_IMAGE_GRAYSCALE);
		sub = inp[i].substr(0,inp[i].length()-4);
		sub += ".yml";

		if (stage > 0) {
			FileStorage file(readName + sub, FileStorage::READ);
			file["result"] >> context;
			file.release();
			output = testClassifier(image, context);
		}
		else {
			output = testClassifier(image, Mat());
		}
		//cout << sub << endl;
		FileStorage file(writeName + sub , cv::FileStorage::WRITE);
		file << "result"  << output;
		file.release();
		output.convertTo(output,CV_8UC1, 255);
		imwrite(writeName + inp[i], output);

		cout << "image " << i << " written" << endl;
	}
}

// It runs the trained classifiers (all the stages) on the test images in the pathI folder and writes them in the output folder.
//Note that this function does not write the outputs for intermediate stages, though, it uses them to compute the final result.
void seriesClassifier::generateResultsTest(const char* pathI) {

	stringstream WriteName;
	WriteName << pathO << "testingOutputs" << "/";
	string writeName  = WriteName.str();
	boost::filesystem::create_directories(writeName);

	stringstream ReadName;
	ReadName << pathO << "trainingOutputsStage" << stage-1 <<"/";
	string readName  = ReadName.str();

	cout << "generating Test results" << endl;
	vector<string> inp = listOfImages(pathI);
	Mat image, context;
	Mat output;
	string sub;


	for (int i = 0; i < inp.size(); ++i) {
		image = imread(pathI + inp[i], CV_LOAD_IMAGE_GRAYSCALE);
		sub = inp[i].substr(0,inp[i].length()-4);
		sub += ".yml";

		for (int j = 0; j < Nstage; ++j) {
			stage = j;
			stringstream sname;
			sname << pathO << "mlp_" << j << ".yml";
			string name = sname.str();
			ann.load(name.c_str(),"ann");
			if (j > 0) {
				output = testClassifier(image, output);
			}
			else {
				output = testClassifier(image, Mat());
			}
		}


		FileStorage file(writeName + sub , cv::FileStorage::WRITE);
		file << "result"  << output;
		file.release();
		output.convertTo(output,CV_8UC1, 255);
		imwrite(writeName + inp[i], output);

		cout << "Test image " << i << " written" << endl;
	}
}

// It saves the trained ANN for each stage. These ANNs will be used at test time.
void seriesClassifier::saveANN() {

	stringstream sname;
	sname << pathO;
	sname << "mlp_";
	sname << stage;
	sname << ".yml";// or xml
	string name = sname.str();
	cv::FileStorage fs(name, cv::FileStorage::WRITE);
	ann.write(*fs, "ann");
	fs.release();
}

void seriesClassifier::setParameters(char* tP, char* lP, char* pO, int rad, int nst, int maxI, int hidN, double eps) {
	trainP = tP;
	labelP = lP;
	pathO = pO;
	radius = rad;
	Nstage = nst;
	maxIter = maxI;
	hiddenN = hidN;
	epsilon = eps;
	boost::filesystem::create_directories(pathO);

}

// This method is responsible to run the whole thing. It trains the classifiers at different stages sequentially.
void seriesClassifier::trainClassifierSeries() {

	for (int i = 0; i < Nstage; ++i) {
		trainClassifier();
		generateResults();
		stage++;
	}
	cout << "***** SERIES TRAINING IS DONE *****" << endl;
}

// The parameters of ANN are set in this method.
void seriesClassifier::setANNParam() {


	criteria.max_iter = 2; // Do not modify this. For maximum iteration change the class member maxIter
	criteria.epsilon  = 0.00001f;
	criteria.type     = CV_TERMCRIT_ITER | CV_TERMCRIT_EPS;

	params.train_method    = CvANN_MLP_TrainParams::BACKPROP;
	params.bp_dw_scale     = epsilon; //0.0001f // step size for gradient descent
	params.bp_moment_scale = 0.1f; // momentum for gradient descent
	params.term_crit       = criteria;

	inputN = (2*radius + 1) * (2*radius + 1);
	inputNC = 2*inputN;
}


