/*
 * seriesClassifier.h
 *
 *  Created on: Apr 1, 2014
 *      Author: mseyed
 */


#ifndef SERIESCLASSIFIER_H_
#define SERIESCLASSIFIER_H_
using namespace std;
using namespace cv;

// Train and test a series of ANN classifiers on input images. Sample usage:
// seriesClassifier SC;
// SC.setParameters(Trainpath, Labelpath, Outputpath, 5, 5, 100);
// SC.trainClassifierSeries();
// SC.generateResultsTest(Tepath);

class seriesClassifier {

public:
	// Constructor
	seriesClassifier();
	// Destructor
	virtual ~seriesClassifier();
	// It extracts patches around all the pixels and return them in the trainData. Each row is a sample in trainData.
	// The format of trainData is ready to pass to ann.
	void extractPatch(const Mat& img, Mat& trainData);
	// It extracts patches around all the pixels for context images. Each row is a sample in traindData.
	virtual void extractPatchContext(const Mat& img, Mat& trainData);
	// Given a label image, it scales it and return a binary matrix of binary labels.
	void extractLabels(const Mat& label, Mat& trainLabel);
	// It trains an ANN.
	void trainClassifier();
	// It handles the sequential training in the series.
	void trainClassifierSeries();
	// It runs a trained ANN on input image. In case of the presence of context it also uses the context.
	Mat testClassifier(const Mat& image, const Mat& context);
	// It generate the output for training images at each stage and writes them in the pathO directory.
	void generateResults();
	// It generates the output of last stage for testing images and writes them in the pathO directory.
	void generateResultsTest(const char* pathTe);
	// It saves the trained ANNs in the pathO directory.
	void saveANN();
	// It sets the basic parameters of ANNs.
	virtual void setANNParam();
	// It set the parameters of the class.
	virtual void setParameters(char* tP, char* lP, char* pO, int rad = 5, int nst = 5, int maxI = 100, int hidN = 10, double eps = 0.0001);
protected:
	// The current stage's ANN.
	CvANN_MLP ann;
	// Parameters of ANN.
	CvANN_MLP_TrainParams params;
	// Stopping criteria for ANN.
	CvTermCriteria criteria;
	// radius of the path used to extract neighborhoods around input pixels.
	int radius;
	// Number of stages in the series.
	int Nstage;
	// Current stage.
	int stage;
	// Number of hidden nodes in the ANN.
	int hiddenN;
	// Number of input nodes in the ANN for the first ANN in the series.
	int inputN;
	// Number of input nodes in the ANN for the second to last stage.
	int inputNC;
	// Maximum number of iterations for the ANN.
	int maxIter;
	// The path to the folder with training images.
	char *trainP;
	// The path to the folder with training labels.
	char *labelP;
	// The path that the outputs are written into.
	char *pathO;
	// The step size for gradient descent
	double epsilon;

};

#endif /* SERIESCLASSIFIER_H_ */
