/*
 * seriesClassifierMS.cpp
 *
 *  Created on: Apr 1, 2014
 *      Author: mseyed
 */
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/ml/ml.hpp"
#include <dirent.h>
#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include<boost/filesystem.hpp>
using namespace std;
using namespace cv;
#include "seriesClassifierMS.h"

seriesClassifierMS::seriesClassifierMS() {
	stage = 0;
	epsilon = 0.0001;
}

seriesClassifierMS::~seriesClassifierMS() {
}

// It provides the offset from the center pixel for a given patch radius.
Mat seriesClassifierMS::getOffset(int rad) {
	int fullsize = 2*rad+1;
	Mat O1 = Mat::ones(fullsize,1, CV_32FC1);
	Mat ind(1,fullsize, CV_32FC1);
	for (int i = 0; i < fullsize; ++i) {
		ind.at<float>(0,i) = -rad + i;
	}
	Mat offX = O1 * ind; // offsets for the first dimension (row wise).

	Mat offY; // offsets for the second dimension (column wise).
	transpose(offX,offY);

	offX = offX.reshape(0,1);
	offY = offY.reshape(0,1);
	Mat offset(2,fullsize*fullsize, CV_32S);
	offX.row(0).copyTo(offset.row(0));
	offY.row(0).copyTo(offset.row(1));
	return offset;
}

// Given an image and corresponding offset (from getOffset method), it extracts neighborhoods for each pixel.
Mat seriesClassifierMS::constructNeighborhood(const Mat& image, Mat& offset){
	double minVal, maxVal;
	minMaxLoc( offset, &minVal, &maxVal);
	Mat PadImg, output;
	int border = (int) maxVal;
	copyMakeBorder(image, PadImg, border, border, border, border, BORDER_REPLICATE); // Pad image to handle pixels close to the border.
	output.create(image.cols * image.rows,(2*radiusC+1)*(2*radiusC+1),CV_32FC1);
	int counter = 0;
	for (int i = border; i < PadImg.rows - border; ++i){
		for (int j = border; j < PadImg.cols - border; ++j){
			for (int k = 0; k < offset.cols; ++k) {
				output.at<float>(counter,k) = PadImg.at<float>(i+offset.at<int>(0,k),j+offset.at<int>(1,k));
			}
			counter++;
		}
	}
	return output;
}

// It extracts patches from context image at multiple scales.
void seriesClassifierMS::extractPatchContext(const Mat& img, Mat& trainData) {

	Mat offset;
	Mat temp, imageC, filt;
	for (int i = 0; i < Nscale; ++i) {
		offset = pow(2,i)*getOffset(radiusC);
		if (i==0){
			trainData = constructNeighborhood(img, offset);
		}
		else {
			double sigmaX = i;
			GaussianBlur( img, imageC, Size(6*sigmaX + 1, 6*sigmaX + 1), sigmaX, sigmaX);
			temp = constructNeighborhood(imageC, offset);
			hconcat(trainData, temp, trainData);
		}
	}
}

void seriesClassifierMS::setParameters(char* tP, char* lP, char* pO, int rad, int nst, int maxI, int hidN, double eps, int Nsc, int radC) {
	trainP = tP;
	labelP = lP;
	pathO = pO;
	radius = rad;
	Nstage = nst;
	boost::filesystem::create_directories(pathO);
	Nscale = Nsc;
	radiusC = radC;
	maxIter = maxI;
	hiddenN = hidN;
	epsilon = eps;

}

void seriesClassifierMS::setANNParam() {

	criteria.max_iter = 2; // Do not modify this. For maximum iteration change the class member maxIter
	criteria.epsilon  = 0.00001f;
	criteria.type     = CV_TERMCRIT_ITER | CV_TERMCRIT_EPS;

	params.train_method    = CvANN_MLP_TrainParams::BACKPROP;
	params.bp_dw_scale     = epsilon; //0.0001f;// step size for gradient descent
	params.bp_moment_scale = 0.1f; // momentum for gradient descent
	params.term_crit       = criteria;

	inputN = (2*radius + 1) * (2*radius + 1);
	inputNC = Nscale*(2*radiusC+1)*(2*radiusC+1) + inputN;
}
