/*
 * seriesClassifierMS.h
 *
 *  Created on: Apr 1, 2014
 *      Author: mseyed
 */

#ifndef SERIESCLASSIFIERMS_H_
#define SERIESCLASSIFIERMS_H_

#include "seriesClassifier.h"

// Train and test a series of ANN classifiers on input images in the MULTI-SCALE mode.
// This class inherits from the seriesClassifier class. Sample usage:
// seriesClassifierMS SC;
// SC.setParameters(Trainpath, Labelpath, Outputpath, 5, 5, 100);
// SC.trainClassifierSeries();
// SC.generateResultsTest(Tepath);

class seriesClassifierMS: public seriesClassifier {
public:
	// Constructor
	seriesClassifierMS();
	// Destructor
	virtual ~seriesClassifierMS();
	// Prepare the offset of the neighborhoods with respect to the center pixel.
	Mat getOffset(int rad);
	// Uses the provided offset and extract neighborhoods for all the pixels.
	Mat constructNeighborhood(const Mat& image, Mat& offset);
	// It extract neighborhood for the context at Multi-scale mode. Note that this method overrides the method from parent class.
	virtual void extractPatchContext(const Mat& img, Mat& trainData);
	// It sets the paramters of ANN. Note that this method overrides the method from parent class.
	virtual void setANNParam();
	// It sets the basic parameters of the class. Note that this method overrides the method from parent class.
	virtual void setParameters(char* tP, char* lP, char* pO, int rad = 5, int nst = 5, int maxI = 100, int hidN = 10, double eps = 0.0001, int Nsc = 5, int radC = 2);

protected:
	// Number of scales used for multi-scale context
	int Nscale;
	// Radius of the patch used to extract multi-scale context
	int radiusC;
};

#endif /* SERIESCLASSIFIERMS_H_ */
